@@ -51,6 +51,8 @@
 #include "gdm-settings-keys.h"
 
 #define INITIAL_SETUP_USERNAME "gnome-initial-setup"
+#define GDM_SESSION_MODE "gdm"
+#define INITIAL_SETUP_SESSION_MODE "initial-setup"
 #define GNOME_SESSION_SESSIONS_PATH DATADIR "/gnome-session/sessions"
 
 extern char **environ;
@@ -70,6 +72,7 @@ struct GdmLaunchEnvironmentPrivate
 
         char           *session_id;
         char           *session_type;
+        char           *session_mode;
         char           *x11_display_name;
         char           *x11_display_seat_id;
         char           *x11_display_device;
@@ -82,6 +85,7 @@ enum {
         PROP_0,
         PROP_VERIFICATION_MODE,
         PROP_SESSION_TYPE,
+        PROP_SESSION_MODE,
         PROP_X11_DISPLAY_NAME,
         PROP_X11_DISPLAY_SEAT_ID,
         PROP_X11_DISPLAY_DEVICE,
@@ -99,6 +103,7 @@ enum {
         STOPPED,
         EXITED,
         DIED,
+        HOSTNAME_SELECTED,
         LAST_SIGNAL
 };
 
@@ -151,6 +156,22 @@ build_launch_environment (GdmLaunchEnvironment *launch_environment,
         if (launch_environment->priv->x11_authority_file != NULL)
                 g_hash_table_insert (hash, g_strdup ("XAUTHORITY"), g_strdup (launch_environment->priv->x11_authority_file));
 
+        if (launch_environment->priv->session_mode != NULL) {
+                g_hash_table_insert (hash, g_strdup ("GNOME_SHELL_SESSION_MODE"), g_strdup (launch_environment->priv->session_mode));
+
+		if (strcmp (launch_environment->priv->session_mode, INITIAL_SETUP_SESSION_MODE) != 0) {
+			/* gvfs is needed for fetching remote avatars in the initial setup. Disable it otherwise. */
+			g_hash_table_insert (hash, g_strdup ("GVFS_DISABLE_FUSE"), g_strdup ("1"));
+			g_hash_table_insert (hash, g_strdup ("GIO_USE_VFS"), g_strdup ("local"));
+			g_hash_table_insert (hash, g_strdup ("GVFS_REMOTE_VOLUME_MONITOR_IGNORE"), g_strdup ("1"));
+
+			/* The locked down dconf profile should not be used for the initial setup session.
+			 * This allows overridden values from the user profile to take effect.
+			 */
+			g_hash_table_insert (hash, g_strdup ("DCONF_PROFILE"), g_strdup ("gdm"));
+		}
+        }
+
         g_hash_table_insert (hash, g_strdup ("LOGNAME"), g_strdup (launch_environment->priv->user_name));
         g_hash_table_insert (hash, g_strdup ("USER"), g_strdup (launch_environment->priv->user_name));
         g_hash_table_insert (hash, g_strdup ("USERNAME"), g_strdup (launch_environment->priv->user_name));
@@ -183,7 +204,6 @@ build_launch_environment (GdmLaunchEnvironment *launch_environment,
         g_hash_table_insert (hash, g_strdup ("PATH"), g_strdup (g_getenv ("PATH")));
 
         g_hash_table_insert (hash, g_strdup ("RUNNING_UNDER_GDM"), g_strdup ("true"));
-        g_hash_table_insert (hash, g_strdup ("DCONF_PROFILE"), g_strdup ("gdm"));
 
         return hash;
 }
@@ -248,6 +268,15 @@ on_session_died (GdmSession           *session,
         g_signal_emit (G_OBJECT (launch_environment), signals [DIED], 0, signal_number);
 }
 
+static void
+on_hostname_selected (GdmSession               *session,
+                      const char               *hostname,
+		      GdmLaunchEnvironment     *launch_environment)
+{
+        g_debug ("GdmSession: hostname selected: %s", hostname);
+        g_signal_emit (launch_environment, signals [HOSTNAME_SELECTED], 0, hostname);
+}
+
 static void
 on_conversation_started (GdmSession           *session,
                          const char           *service_name,
@@ -256,7 +285,11 @@ on_conversation_started (GdmSession           *session,
         char             *log_path;
         char             *log_file;
 
-        log_file = g_strdup_printf ("%s-greeter.log", launch_environment->priv->x11_display_name);
+        if (launch_environment->priv->x11_display_name != NULL)
+                log_file = g_strdup_printf ("%s-greeter.log", launch_environment->priv->x11_display_name);
+        else
+                log_file = g_strdup ("greeter.log");
+
         log_path = g_build_filename (LOGDIR, log_file, NULL);
         g_free (log_file);
 
@@ -394,6 +427,11 @@ gdm_launch_environment_start (GdmLaunchEnvironment *launch_environment)
                                  G_CALLBACK (on_session_died),
                                  launch_environment,
                                  0);
+        g_signal_connect_object (launch_environment->priv->session,
+                                 "hostname-selected",
+                                 G_CALLBACK (on_hostname_selected),
+                                 launch_environment,
+                                 0);
 
         gdm_session_start_conversation (launch_environment->priv->session, "gdm-launch-environment");
         gdm_session_select_program (launch_environment->priv->session, launch_environment->priv->command);
@@ -459,6 +497,14 @@ _gdm_launch_environment_set_session_type (GdmLaunchEnvironment *launch_environme
         launch_environment->priv->session_type = g_strdup (session_type);
 }
 
+static void
+_gdm_launch_environment_set_session_mode (GdmLaunchEnvironment *launch_environment,
+                                          const char           *session_mode)
+{
+        g_free (launch_environment->priv->session_mode);
+        launch_environment->priv->session_mode = g_strdup (session_mode);
+}
+
 static void
 _gdm_launch_environment_set_x11_display_name (GdmLaunchEnvironment *launch_environment,
                                               const char           *name)
@@ -547,6 +593,9 @@ gdm_launch_environment_set_property (GObject      *object,
         case PROP_SESSION_TYPE:
                 _gdm_launch_environment_set_session_type (self, g_value_get_string (value));
                 break;
+        case PROP_SESSION_MODE:
+                _gdm_launch_environment_set_session_mode (self, g_value_get_string (value));
+                break;
         case PROP_X11_DISPLAY_NAME:
                 _gdm_launch_environment_set_x11_display_name (self, g_value_get_string (value));
                 break;
@@ -597,6 +646,9 @@ gdm_launch_environment_get_property (GObject    *object,
         case PROP_SESSION_TYPE:
                 g_value_set_string (value, self->priv->session_type);
                 break;
+        case PROP_SESSION_MODE:
+                g_value_set_string (value, self->priv->session_mode);
+                break;
         case PROP_X11_DISPLAY_NAME:
                 g_value_set_string (value, self->priv->x11_display_name);
                 break;
@@ -656,6 +708,13 @@ gdm_launch_environment_class_init (GdmLaunchEnvironmentClass *klass)
                                                               NULL,
                                                               NULL,
                                                               G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_SESSION_MODE,
+                                         g_param_spec_string ("session-mode",
+                                                              NULL,
+                                                              NULL,
+                                                              NULL,
+                                                              G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
                                          PROP_X11_DISPLAY_NAME,
                                          g_param_spec_string ("x11-display-name",
@@ -771,6 +830,18 @@ gdm_launch_environment_class_init (GdmLaunchEnvironmentClass *klass)
                               G_TYPE_NONE,
                               1,
                               G_TYPE_INT);
+
+        signals [HOSTNAME_SELECTED] =
+                g_signal_new ("hostname-selected",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmLaunchEnvironmentClass, hostname_selected),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1,
+                              G_TYPE_STRING);
 }
 
 static void
@@ -821,6 +892,7 @@ create_gnome_session_environment (const char *session_id,
                                   const char *display_name,
                                   const char *seat_id,
                                   const char *session_type,
+                                  const char *session_mode,
                                   const char *display_hostname,
                                   gboolean    display_is_local)
 {
@@ -833,7 +905,7 @@ create_gnome_session_environment (const char *session_id,
         gdm_settings_direct_get_boolean (GDM_KEY_DEBUG, &debug);
 
         args = g_ptr_array_new ();
-        g_ptr_array_add (args, BINDIR "/gnome-session");
+        g_ptr_array_add (args, "gnome-session");
 
         g_ptr_array_add (args, "--autostart");
         g_ptr_array_add (args, DATADIR "/gdm/greeter/autostart");
@@ -857,6 +929,7 @@ create_gnome_session_environment (const char *session_id,
                                            "command", command,
                                            "user-name", user_name,
                                            "session-type", session_type,
+                                           "session-mode", session_mode,
                                            "x11-display-name", display_name,
                                            "x11-display-seat-id", seat_id,
                                            "x11-display-hostname", display_hostname,
@@ -877,16 +950,12 @@ gdm_create_greeter_launch_environment (const char *display_name,
 {
         const char *session_name = NULL;
 
-        if (g_strcmp0 (session_type, "wayland") == 0)
-                session_name = "gnome-wayland";
-        else if (g_strcmp0 (session_type, "x11") == 0)
-                session_name = "gnome";
-
         return create_gnome_session_environment (session_name,
                                                  GDM_USERNAME,
                                                  display_name,
                                                  seat_id,
                                                  session_type,
+                                                 GDM_SESSION_MODE,
                                                  display_hostname,
                                                  display_is_local);
 }
@@ -902,6 +971,7 @@ gdm_create_initial_setup_launch_environment (const char *display_name,
                                                  display_name,
                                                  seat_id,
                                                  NULL,
+                                                 INITIAL_SETUP_SESSION_MODE,
                                                  display_hostname,
                                                  display_is_local);
 }
