@@ -61,21 +61,8 @@
 
 static void
 record_set_username (UTMP       *u,
-                     const char *user_name)
+                     const char *username)
 {
-        const char *username;
-
-        /*
-         * It is possible that PAM failed before it mapped the user
-         * input into a valid username, so we fallback to try using
-         * "(unknown)"
-         */
-        if (user_name != NULL) {
-                username = user_name;
-        } else {
-                username = "(unknown)";
-        }
-
 #if defined(HAVE_UT_UT_USER)
         strncpy (u->ut_user,
                  username,
@@ -226,7 +213,9 @@ gdm_session_record_login (GPid                  session_pid,
         /* Handle utmp */
 #if defined(HAVE_GETUTXENT)
         g_debug ("Adding or updating utmp record for login");
+        setutxent();
         pututxline (&session_record);
+        endutxent();
 #elif defined(HAVE_LOGIN)
 	login (&session_record);
 #endif
@@ -269,7 +258,9 @@ gdm_session_record_logout (GPid                  session_pid,
         /* Handle utmp */
 #if defined(HAVE_GETUTXENT)
         g_debug ("Adding or updating utmp record for logout");
+        setutxent();
         pututxline (&session_record);
+        endutxent();
 #elif defined(HAVE_LOGOUT)
         logout (session_record.ut_line);
 #endif
