@@ -144,12 +144,6 @@ write_pid (void)
         atexit (delete_pid);
 }
 
-static void
-delete_first_run_marker (void)
-{
-        g_unlink (GDM_RAN_ONCE_MARKER_FILE);
-}
-
 static gboolean
 ensure_dir_with_perms (const char *path,
                        uid_t       uid,
@@ -226,7 +220,7 @@ gdm_daemon_lookup_user (uid_t *uidp,
 
         /* Set uid and gid */
         if G_UNLIKELY (pwent == NULL) {
-                gdm_fail (_("Can't find the GDM user '%s'. Aborting!"), username);
+                gdm_fail (_("Can’t find the GDM user “%s”. Aborting!"), username);
         } else {
                 uid = pwent->pw_uid;
         }
@@ -238,7 +232,7 @@ gdm_daemon_lookup_user (uid_t *uidp,
         grent = getgrnam (groupname);
 
         if G_UNLIKELY (grent == NULL) {
-                gdm_fail (_("Can't find the GDM group '%s'. Aborting!"), groupname);
+                gdm_fail (_("Can’t find the GDM group “%s”. Aborting!"), groupname);
         } else  {
                 gid = grent->gr_gid;
         }
@@ -315,7 +309,6 @@ main (int    argc,
         GMainLoop          *main_loop;
         GOptionContext     *context;
         GError             *error = NULL;
-        int                 ret;
         gboolean            res;
         static gboolean     do_timed_exit    = FALSE;
         static gboolean     print_version    = FALSE;
@@ -334,24 +327,28 @@ main (int    argc,
         textdomain (GETTEXT_PACKAGE);
         setlocale (LC_ALL, "");
 
-        ret = 1;
-
         context = g_option_context_new (_("GNOME Display Manager"));
         g_option_context_add_main_entries (context, entries, NULL);
-        g_option_context_set_ignore_unknown_options (context, TRUE);
 
         error = NULL;
         res = g_option_context_parse (context, &argc, &argv, &error);
         g_option_context_free (context);
         if (! res) {
-                g_warning ("%s", error->message);
+                g_printerr ("Failed to parse options: %s\n", error->message);
                 g_error_free (error);
-                goto out;
+                return EXIT_FAILURE;
         }
 
         if (print_version) {
                 g_print ("GDM %s\n", VERSION);
-                exit (1);
+                return EXIT_SUCCESS;
+        }
+
+        /* XDM compliant error message */
+        if (getuid () != 0) {
+                /* make sure the pid file doesn't get wiped */
+                g_printerr ("%s\n", _("Only the root user can run GDM"));
+                return EXIT_FAILURE;
         }
 
         if (fatal_warnings) {
@@ -365,14 +362,9 @@ main (int    argc,
         gdm_log_init ();
 
         settings = gdm_settings_new ();
-        if (settings == NULL) {
-                g_warning ("Unable to initialize settings");
-                goto out;
-        }
-
         if (! gdm_settings_direct_init (settings, DATADIR "/gdm/gdm.schemas", "/")) {
                 g_warning ("Unable to initialize settings");
-                goto out;
+                return EXIT_FAILURE;
         }
 
         gdm_log_set_debug (is_debug_set ());
@@ -381,13 +373,6 @@ main (int    argc,
 
         gdm_daemon_ensure_dirs (gdm_uid, gdm_gid);
 
-        /* XDM compliant error message */
-        if (getuid () != 0) {
-                /* make sure the pid file doesn't get wiped */
-                g_warning (_("Only the root user can run GDM"));
-                exit (-1);
-        }
-
         /* Connect to the bus, own the name and start the manager */
         bus_reconnect ();
 
@@ -395,9 +380,6 @@ main (int    argc,
         delete_pid ();
         write_pid ();
 
-        /* clean up any stale ran once marker file that may be lingering */
-        delete_first_run_marker ();
-
         g_chdir ("/");
 
         main_loop = g_main_loop_new (NULL, FALSE);
@@ -422,14 +404,7 @@ main (int    argc,
 
         g_main_loop_unref (main_loop);
 
-        ret = 0;
-
- out:
-        if (error) {
-                g_printerr ("%s\n", error->message);
-                g_clear_error (&error);
-        }
-        return ret;
+        return EXIT_SUCCESS;
 }
 
 static void
@@ -443,13 +418,11 @@ on_name_acquired (GDBusConnection *bus,
         manager = gdm_manager_new ();
         if (manager == NULL) {
                 g_warning ("Could not construct manager object");
-                exit (1);
+                exit (EXIT_FAILURE);
         }
 
         g_debug ("Successfully connected to D-Bus");
 
-        gdm_manager_start (manager);
-
         show_local_greeter = TRUE;
         gdm_settings_direct_get_boolean (GDM_KEY_SHOW_LOCAL_GREETER, &show_local_greeter);
         gdm_manager_set_show_local_greeter (manager, show_local_greeter);
@@ -457,6 +430,8 @@ on_name_acquired (GDBusConnection *bus,
         xdmcp_enabled = FALSE;
         gdm_settings_direct_get_boolean (GDM_KEY_XDMCP_ENABLE, &xdmcp_enabled);
         gdm_manager_set_xdmcp_enabled (manager, xdmcp_enabled);
+
+        gdm_manager_start (manager);
 }
 
 static void
